from django.forms import ModelForm, ValidationError, TextInput, Form, URLField, MultipleHiddenInput, Textarea, CharField
from datetime import date
from django_better_admin_arrayfield.forms.widgets import DynamicArrayWidget

from .models import Event, BatchImportation
from django.utils.translation import gettext_lazy as _

class EventSubmissionForm(Form):
    url = URLField(max_length=512)


class DynamicArrayWidgetURLs(DynamicArrayWidget):
    template_name = "agenda_culturel/widgets/widget-urls.html"

class DynamicArrayWidgetTags(DynamicArrayWidget):
    template_name = "agenda_culturel/widgets/widget-tags.html"

class EventForm(ModelForm):

    class Meta:
        model = Event
        exclude = ["possibly_duplicated", "imported_date", "modified_date"]
        widgets = {
            'start_day': TextInput(attrs={'type': 'date', 'onchange': 'update_datetimes(event);', "onfocus": "this.oldvalue = this.value;"}),
            'start_time': TextInput(attrs={'type': 'time', 'onchange': 'update_datetimes(event);', "onfocus": "this.oldvalue = this.value;"}),
            'end_day': TextInput(attrs={'type': 'date'}),
            'end_time': TextInput(attrs={'type': 'time'}),
            'uuids': MultipleHiddenInput(),
            'import_sources': MultipleHiddenInput(),
            'reference_urls': DynamicArrayWidgetURLs(),
            'tags': DynamicArrayWidgetTags(),
        }


    def __init__(self, *args, **kwargs):
        is_authenticated = kwargs.pop('is_authenticated', False)
        super().__init__(*args, **kwargs)
        if not is_authenticated:
            del self.fields['status']


    def clean_end_day(self):
        start_day = self.cleaned_data.get("start_day")
        end_day = self.cleaned_data.get("end_day")

        if end_day is not None and start_day is not None and end_day < start_day:
            raise ValidationError(_("The end date must be after the start date."))

        return end_day

    def clean_end_time(self):
        start_day = self.cleaned_data.get("start_day")
        end_day = self.cleaned_data.get("end_day")
        start_time = self.cleaned_data.get("start_time")
        end_time = self.cleaned_data.get("end_time")

        # same day
        if start_day is not None and (end_day is None or start_day == end_day):
            # both start and end time are defined
            if start_time is not None and end_time is not None:
                if start_time > end_time:
                    raise ValidationError(_("The end time cannot be earlier than the start time."))

        return end_time



class BatchImportationForm(ModelForm):

    json = CharField(label="JSON (facultatif)", widget=Textarea(attrs={"rows":"10"}), help_text=_("JSON in the format expected for the import. If the JSON is provided here, we will ignore the URLs given above, and use the information provided by the json without importing any additional events from the URL."), required=False)

    class Meta:
        model = BatchImportation
        fields = ['source', 'browsable_url']

    def clean(self):
        cleaned_data = super().clean()
        json = cleaned_data.get("json")
        source = cleaned_data.get("source")
        browsable_url = cleaned_data.get("browsable_url")

        if (not json or json == "") and (not source or source == "") and (not browsable_url or browsable_url == ""):
            raise ValidationError(_("You need to fill in either the json or the source possibly supplemented by the navigable URL."))

        # Always return a value to use as the new cleaned data, even if
        # this method didn't change it.
        return cleaned_data


